//
//  Persistence.swift
//  SwiftUICoreDataSpendingTrackerLBTA
//
//  Created by Brian Voong on 6/27/21.
//

import CoreData
import UIKit

struct PersistenceController {
    static let shared = PersistenceController()

    static var preview: PersistenceController = {
        let result = PersistenceController(inMemory: true)
        let viewContext = result.container.viewContext
        for _ in 0..<10 {
            let newItem = Item(context: viewContext)
            newItem.timestamp = Date()
        }
        do {
            try viewContext.save()
        } catch {
            // Replace this implementation with code to handle the error appropriately.
            // fatalError() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
            let nsError = error as NSError
            fatalError("Unresolved error \(nsError), \(nsError.userInfo)")
        }
        return result
    }()

    let container: NSPersistentContainer

    init(inMemory: Bool = false) {
        container = NSPersistentContainer(name: "SwiftUICoreDataSpendingTrackerLBTA")
        if inMemory {
            container.persistentStoreDescriptions.first!.url = URL(fileURLWithPath: "/dev/null")
        }
        container.loadPersistentStores(completionHandler: { (storeDescription, error) in
            if let error = error as NSError? {
                // Replace this implementation with code to handle the error appropriately.
                // fatalError() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.

                /*
                Typical reasons for an error here include:
                * The parent directory does not exist, cannot be created, or disallows writing.
                * The persistent store is not accessible, due to permissions or data protection when the device is locked.
                * The device is out of space.
                * The store could not be migrated to the current model version.
                Check the error message to determine what the actual problem was.
                */
                fatalError("Unresolved error \(error), \(error.userInfo)")
            }
        })
        
        seedInitialData()
    }
    
    private func seedInitialData() {
        if UserDefaults.standard.bool(forKey: Self.hasSeededDataKey) {
            return
        }
        
        let context = container.viewContext
        
        let category = TransactionCategory(context: context)
        category.name = "Office Supplies"
        category.colorData = UIColor.blue.encode()
        category.timestamp = Date()
        
        do {
            try context.save()
            UserDefaults.standard.set(true, forKey: Self.hasSeededDataKey)
        } catch {
            print("Failed to seed initial data:", error)
        }
    }
    
    static let hasSeededDataKey = "hasSeededDataKey"
}
